# Parent vs Owner
# http://blogs.msdn.com/b/oldnewthing/archive/2010/03/15/9978691.aspx

require 'TT_Lib2/win32/api'

GA_PARENT     = 1
GA_ROOT       = 2
GA_ROOTOWNER  = 3

GetAncestor = Win32::API.new('GetAncestor' , 'LI', 'L', 'user32')

GetWindowText       = Win32::API.new('GetWindowText', 'LPI', 'I', 'user32')
GetWindowTextLength = Win32::API.new('GetWindowTextLength', 'L', 'I', 'user32')

# http://stackoverflow.com/questions/3327666/win32s-findwindow-can-find-a-particular-window-with-the-exact-title-but-what
EnumWindows       = Win32::API.new('EnumWindows', 'KP', 'L', 'user32')
EnumThreadWindows = Win32::API.new('EnumThreadWindows', 'LKP', 'I', 'user32')

GetCurrentThreadId = Win32::API.new('GetCurrentThreadId', '', 'L', 'kernel32')


# If one creates too many Win32::API::Callbacks one get the following error:
# +Error: #<Win32::API::Error: too many callbacks are defined.>+ This presents
# a problem when you need to enumerate windows.
#
# Use this class to avoid this.
#
#  param = 'SketchUpMainWindow'
#  enumWindowsProc = EnumWindowsProc.new(param) { |handle|
#    # Return 0 (FALSE) to stop enumeration or 1 (TRUE) to proceed.
#  }
#  EnumThreadWindows.call(threadId, enumWindowsProc.callback, param)
class EnumWindowsProc
  
  attr_reader( :param )
  
  @@enumThreadWndProc = Win32::API::Callback.new('LP', 'I'){ |hwnd, param|
    if @@callbacks.key?(param)
      @@callbacks[param].call(hwnd)
    end
  }
  @@callbacks = {}
  
  # @param [String] param - the +param+ argument this callback should repond to.
  def initialize(param, &block)
    @param = param.dup
    @@callbacks[param] = block
  end
  
  def callback
    @@enumThreadWndProc
  end
  
  def destroy_callback(param)
    @@callbacks.delete?(param)
  end
  
end # class EnumThreadWndProc


# Returns the window text for the given window.
def get_window_text(hwnd)
  buf_len = GetWindowTextLength.call(hwnd)
  str = ' ' * (buf_len + 1)
  GetWindowText.call(hwnd, str, str.length)
  str
end


# Returns the handle to the SketchUp window the calling thread belongs to.
# 
# NOTE: returned string has a NULL character at the end.
def find_sketchup_window
  threadId = GetCurrentThreadId.call
  hwnd = 0
  param = 'SketchUpMainWindow'
  enumWindowsProc = EnumWindowsProc.new(param) { |handle|
    hwnd = GetAncestor.call(handle, GA_ROOTOWNER)
    0
  }
  EnumThreadWindows.call(threadId, enumWindowsProc.callback, param)
  hwnd
end


# Test method for find_sketchup_window. Delays the execution so one can test
# the results under various conditions, such as having focus on different
# windows or applciations.
def test_find_sketchup_window(delay=1)
  UI.start_timer(delay, false) {
    hwnd = find_sketchup_window
    p get_window_text(hwnd).rstrip
  }
end